@@ -1,5 +1,5 @@
 import numpy as np
-import pandas as pd
+import sys
 
 from matplotlib import pyplot as plt, gridspec, patches, rc_context, cm, colors
 from astropy.wcs import WCS
@@ -10,7 +10,7 @@ from collections import Sequence
 from pathlib import Path
 
 from .model import c_KMpS, gaussian_profiles_computation, linear_continuum_computation, format_line_mask_option
-from .tools import label_decomposition, kinematic_component_labelling, blended_label_from_log
+from .tools import label_decomposition, blended_label_from_log, UNITS_LATEX_DICT, latex_science_float
 
 try:
     import pylatex
@@ -65,15 +65,6 @@ STANDARD_PLOT = {**PLOT_SIZE_FONT, **PLOT_COLORS}
 STANDARD_AXES = {'xlabel': r'Wavelength $(\AA)$', 'ylabel': r'Flux $(erg\,cm^{-2} s^{-1} \AA^{-1})$'}
 
 
-def latex_science_float(f, dec=2):
-    float_str = f'{f:.{dec}g}'
-    if "e" in float_str:
-        base, exponent = float_str.split("e")
-        return r"{0} \times 10^{{{1}}}".format(base, int(exponent))
-    else:
-        return float_str
-
-
 def format_for_table(entry, rounddig=4, rounddig_er=2, scientific_notation=False, nan_format='none'):
 
     if rounddig_er == None: #TODO declare a universal tool
@@ -127,7 +118,7 @@ def format_for_table(entry, rounddig=4, rounddig_er=2, scientific_notation=False
     return formatted_entry
 
 
-def table_fluxes(lines_df, table_address, header_format_latex, table_type='pdf', fit_conf={}):
+def table_fluxes(lines_df, table_address, header_format_latex, table_type='pdf', lines_notation=None):
 
     # Check pylatex is install else leave
     if pylatex_check:
@@ -148,7 +139,7 @@ def table_fluxes(lines_df, table_address, header_format_latex, table_type='pdf',
         idx_blended_label = None
 
     # Get the line latex label for the table
-    ion_array, wavelength_array, latexLabel_array = label_decomposition(lines_df.index.values, comp_dict=fit_conf)
+    # ion_array, wavelength_array, latexLabel_array = label_decomposition(lines_df.index.values, comp_dict=fit_conf, units_wave=self.units_wave)
 
     # Create pdf
     pdf = PdfMaker()
@@ -158,7 +149,7 @@ def table_fluxes(lines_df, table_address, header_format_latex, table_type='pdf',
     # Loop through the lines
     obsLines = lines_df.index.values
     for i, lineLabel in enumerate(obsLines):
-        row_raw = [latexLabel_array[i]] + list(lines_df.loc[lineLabel].values)
+        row_raw = [lines_notation[i]] + list(lines_df.loc[lineLabel].values)
 
         # Exclude the _ from the blended label
         if idx_blended_label is not None:
@@ -188,7 +179,7 @@ def numberStringFormat(value, cifras=4):
     return newFormat
 
 
-def mplcursors_legend(line, log, latex_label, norm_flux):
+def mplcursors_legend(line, log, latex_label, norm_flux, units_wave):
 
     if len(latex_label) == 1:
         legend_text = latex_label[0] + '\n'
@@ -197,7 +188,7 @@ def mplcursors_legend(line, log, latex_label, norm_flux):
         legend_text = '+'.join(latex_label) + '\n'
 
     else:
-        ion, wave, latex = label_decomposition(line, scalar_output=True)
+        ion, wave, latex = label_decomposition(line, scalar_output=True, units_wave=units_wave)
         legend_text = latex + '\n'
 
     intg_flux = latex_science_float(log.loc[line, 'intg_flux']/norm_flux)
@@ -380,6 +371,7 @@ def spatial_mask_generator(mask_param, wavelength_array, flux_cube, contour_leve
             ax.legend(handles=legend_list, loc=2)
             ax.set(**AXES_CONF)
 
+            # Save the mask image
             if fits_folder is not None:
 
                 if mask_ref is None:
@@ -389,12 +381,12 @@ def spatial_mask_generator(mask_param, wavelength_array, flux_cube, contour_leve
 
                 plt.savefig(output_image)
 
+            # Display the mask if requested
             if show_plot:
-                plt.show()
+                save_close_fig_swicth(None, 'tight', fig)
 
-            plt.close(fig)
 
-    # Save to a fits file:
+    # Save the mask to a fits file:
     if output_address is not None:
 
         fits_address = Path(output_address)
@@ -430,6 +422,30 @@ def spatial_mask_generator(mask_param, wavelength_array, flux_cube, contour_leve
     return
 
 
+def save_close_fig_swicth(file_path=None, bbox_inches=None, fig_obj=None):
+
+    # By default plot on screen unless an output address is provided
+    if file_path is None:
+
+        if bbox_inches is not None:
+            plt.tight_layout()
+
+        plt.show()
+
+    else:
+        plt.savefig(file_path, bbox_inches=bbox_inches)
+
+        # Close the figure in the case of printing
+        if fig_obj is not None:
+            plt.close(fig_obj)
+
+    # # Close the figure before leaving
+    # if plt.get_backend() not in ["Qt5Agg", "nbagg"]:
+    #     plt.close(fig)
+
+    return
+
+
 class LiMePlots:
 
     def __init__(self):
@@ -498,7 +514,11 @@ class LiMePlots:
         # Adjust default theme
         PLOT_CONF = STANDARD_PLOT.copy()
         AXES_CONF = STANDARD_AXES.copy()
+
         PLOT_CONF['figure.figsize'] = (10, 6)
+        norm_label = r' $\,/\,{}$'.format(latex_science_float(self.norm_flux)) if self.norm_flux != 1.0 else ''
+        AXES_CONF['ylabel'] = f'Flux $({UNITS_LATEX_DICT[self.units_flux]})$' + norm_label
+        AXES_CONF['xlabel'] = f'Wavelength $({UNITS_LATEX_DICT[self.units_wave]})$'
 
         # User configuration overrites user
         PLT_CONF = {**PLOT_CONF, **plt_cfg}
@@ -536,7 +556,7 @@ class LiMePlots:
 
             # Shade regions of matched lines if provided
             if match_log is not None:
-                ion_array, wave_array, latex_array = label_decomposition(match_log.index.values)
+                ion_array, wave_array, latex_array = label_decomposition(match_log.index.values, units_wave=self.units_wave)
                 w3, w4 = match_log.w3.values * (1+self.redshift), match_log.w4.values * (1+self.redshift)
                 idcsLineBand = np.searchsorted(wave_plot, np.array([w3, w4]))
 
@@ -619,10 +639,6 @@ class LiMePlots:
             if log_scale:
                 ax.set_yscale('log')
 
-            # Add the axis normalization to the flux units if non provided
-            if self.norm_flux != 1.0:
-                AXES_CONF['ylabel'] = AXES_CONF['ylabel'] + r' $\,/\,{}$'.format(latex_science_float(self.norm_flux))
-
             # Add the figure labels
             ax.set(**AXES_CONF)
 
@@ -631,14 +647,7 @@ class LiMePlots:
                 ax.legend()
 
             # By default plot on screen unless an output address is provided
-            if output_address is None:
-                plt.tight_layout()
-                plt.show()
-            else:
-                plt.savefig(output_address, bbox_inches='tight')
-
-            # Close the figure before leaving
-            plt.close(fig)
+            save_close_fig_swicth(output_address, 'tight', fig)
 
         return
 
@@ -658,7 +667,9 @@ class LiMePlots:
         PLT_CONF = STANDARD_PLOT.copy()
         AXES_CONF = STANDARD_AXES.copy()
         PLT_CONF['axes.labelsize'] = 14
-        AXES_CONF.pop('xlabel')
+        norm_label = r' $\,/\,{}$'.format(latex_science_float(self.norm_flux)) if self.norm_flux != 1.0 else ''
+        AXES_CONF['ylabel'] = f'Flux $({UNITS_LATEX_DICT[self.units_flux]})$' + norm_label
+        AXES_CONF['xlabel'] = f'Wavelength $({UNITS_LATEX_DICT[self.units_wave]})$'
 
         # User configuration overrites user
         PLT_CONF = {**PLT_CONF, **plt_cfg}
@@ -669,13 +680,23 @@ class LiMePlots:
             # List the profile components
             list_comps = profile_label.split('-') if blended_check else [line]
 
+            # In case the input list of lines is different from the order stored in the logs, the latter takes precedence
+            if blended_check:
+                log_list = list(self.log.loc[self.log.profile_label == profile_label].index.values)
+                list_comps = log_list if not np.array_equal(list_comps, log_list) else list_comps
+
             # Reference frame for the plot
             wave_plot, flux_plot, z_corr, idcs_mask = self.frame_mask_switch(self.wave, self.flux, self.redshift, frame)
 
-            # Determine the line region
+            # Determine the line region # WARNING it needs to be a bit larger than original mask
             w1 = self.log.loc[line, 'w1'] * (1 + self.redshift)
             w6 = self.log.loc[line, 'w6'] * (1 + self.redshift)
-            idcs_plot = ((w1 - 5) <= wave_plot) & (wave_plot <= (w6 + 5))
+            idx1, idx6 = np.searchsorted(wave_plot, (w1, w6))
+            idcs_plot = np.full(wave_plot.size, False)
+
+            idx_low = idx1 - 1 if idx1-1 >= 0 else idx1
+            idx_high = idx6 + 1 if idx6+1 < wave_plot.size else idx6
+            idcs_plot[idx_low:idx_high] = True
 
             # Continuum level
             cont_level = self.log.loc[line, 'cont']
@@ -731,11 +752,6 @@ class LiMePlots:
                 resid_ax.fill_between(wave_plot[idcs_plot]/z_corr, -err_norm*z_corr, err_norm*z_corr, label=label,
                                       facecolor='salmon', alpha=0.3)
 
-            # Add the flux normalization to units if non provided
-            if self.norm_flux != 1.0:
-                norm_label = AXES_CONF['ylabel'] + r' $\,/\,{}$'.format(latex_science_float(self.norm_flux))
-                AXES_CONF['ylabel'] = norm_label
-
             # Switch y_axis to logarithmic scale if requested
             if log_scale:
                 spec_ax.set_yscale('log')
@@ -746,6 +762,7 @@ class LiMePlots:
 
             # Add the figure labels
             spec_ax.set(**AXES_CONF)
+            spec_ax.set_xlabel(None)
             spec_ax.legend()
 
             # Spec upper and lower limit based on absorption or emission
@@ -765,14 +782,10 @@ class LiMePlots:
             # Residual plot labeling
             resid_ax.legend(loc='upper left')
             resid_ax.set_ylabel(label_residual, fontsize=22)
-            resid_ax.set_xlabel(r'Wavelength $(\AA)$')
+            resid_ax.set_xlabel(AXES_CONF['xlabel'])
 
             # By default plot on screen unless an output address is provided
-            if output_address is None:
-                plt.tight_layout()
-                plt.show()
-            else:
-                plt.savefig(output_address, bbox_inches='tight')
+            save_close_fig_swicth(output_address, 'tight', fig_obj=None)
 
         return
 
@@ -788,6 +801,8 @@ class LiMePlots:
         # Adjust default theme
         PLOT_CONF = STANDARD_PLOT.copy()
         AXES_CONF = STANDARD_AXES.copy()
+        norm_label = r' $\,/\,{}$'.format(latex_science_float(self.norm_flux)) if self.norm_flux != 1.0 else ''
+        AXES_CONF['ylabel'] = f'Flux $({UNITS_LATEX_DICT[self.units_flux]})$' + norm_label
         AXES_CONF['xlabel'] = 'Velocity (Km/s)'
 
         # User configuration overrites user
@@ -888,23 +903,12 @@ class LiMePlots:
             if log_scale:
                 ax.set_yscale('log')
 
-            # Add the axis normalization to the flux units if non provided
-            if self.norm_flux != 1.0:
-                AXES_CONF['ylabel'] = AXES_CONF['ylabel'] + r' $\,/\,{}$'.format(latex_science_float(self.norm_flux))
-
             # Add the figure labels
             ax.set(**AXES_CONF)
             ax.legend()
 
             # By default plot on screen unless an output address is provided
-            if output_address is None:
-                plt.tight_layout()
-                plt.show()
-            else:
-                plt.savefig(output_address, bbox_inches='tight')
-
-            # Close the figure before leaving
-            plt.close(fig)
+            save_close_fig_swicth(output_address, 'tight', fig)
 
         return
 
@@ -912,7 +916,7 @@ class LiMePlots:
 
         # Line labels to plot
         line_list = log.index.values
-        ion_array, wave_array, latex_array = label_decomposition(line_list)
+        ion_array, wave_array, latex_array = label_decomposition(line_list, units_wave=self.units_wave)
 
         # Define plot axes grid size
         if nrows is None:
@@ -989,13 +993,7 @@ class LiMePlots:
                     mplcursors.cursor(lineProfile).connect("add", lambda sel, label=label: sel.annotation.set_text(label))
 
             # By default plot on screen unless an output address is provided
-            if output_address is None:
-                plt.tight_layout()
-                plt.show()
-            else:
-                plt.savefig(output_address, bbox_inches='tight')
-
-            plt.close(fig)
+            save_close_fig_swicth(output_address, 'tight', fig)
 
         return
 
@@ -1044,7 +1042,7 @@ class LiMePlots:
 
             idcs_lines = log.index.isin(list_comps)
             observations_list = log.loc[idcs_lines, 'observations'].values
-            ion_array, wavelength_array, latex_array = label_decomposition(list_comps)
+            ion_array, wavelength_array, latex_array = label_decomposition(list_comps, units_wave=self.units_wave)
 
             # Plot the continuum,  Usine wavelength array and continuum form the first component
             cont_wave = wave_array[:, 0]
@@ -1075,7 +1073,7 @@ class LiMePlots:
 
                 # Compute mplcursors box text
                 if mplcursors_check:
-                    label_complex = mplcursors_legend(line, log, latex_array, self.norm_flux)
+                    label_complex = mplcursors_legend(line, log, latex_array, self.norm_flux, self.units_wave)
                     self._legends_dict[label_complex] = line_g
 
             # Combined profile if applicable
